@@ -36,7 +36,6 @@
 
 #include <boost/algorithm/string.hpp>
 #include <boost/algorithm/string/replace.hpp>
-#include <boost/lexical_cast.hpp>
 
 #include <moveit/rviz_plugin_render_tools/trajectory_visualization.h>
 
@@ -57,6 +56,8 @@
 #include <rviz_default_plugins/robot/robot_link.hpp>
 #include <rviz_common/window_manager_interface.hpp>
 
+#include <string>
+
 using namespace std::placeholders;
 
 namespace moveit_rviz_plugin
@@ -108,6 +109,10 @@ TrajectoryVisualization::TrajectoryVisualization(rviz_common::properties::Proper
   state_display_time_property_->addOptionStd("0.1s");
   state_display_time_property_->addOptionStd("0.5s");
 
+  use_sim_time_property_ = new rviz_common::properties::BoolProperty(
+      "Use Sim Time", false, "Indicates whether simulation time or wall-time is used for state display timing.", widget,
+      nullptr, this);
+
   loop_display_property_ = new rviz_common::properties::BoolProperty("Loop Animation", false,
                                                                      "Indicates whether the last received path "
                                                                      "is to be animated in a loop",
@@ -179,8 +184,8 @@ void TrajectoryVisualization::onInitialize(const rclcpp::Node::SharedPtr& node,
   }
 
   trajectory_topic_sub_ = node_->create_subscription<moveit_msgs::msg::DisplayTrajectory>(
-      trajectory_topic_property_->getStdString(), 2,
-      std::bind(&TrajectoryVisualization::incomingDisplayTrajectory, this, _1));
+      trajectory_topic_property_->getStdString(), rclcpp::SystemDefaultsQoS(),
+      [this](const moveit_msgs::msg::DisplayTrajectory::ConstSharedPtr& msg) { return incomingDisplayTrajectory(msg); });
 }
 
 void TrajectoryVisualization::setName(const QString& name)
@@ -251,12 +256,13 @@ void TrajectoryVisualization::changedShowTrail()
 
   int stepsize = trail_step_size_property_->getInt();
   // always include last trajectory point
-  trajectory_trail_.resize(static_cast<int>(std::ceil((t->getWayPointCount() + stepsize - 1) / (float)stepsize)));
+  trajectory_trail_.resize(
+      static_cast<int>(std::ceil((t->getWayPointCount() + stepsize - 1) / static_cast<float>(stepsize))));
   for (std::size_t i = 0; i < trajectory_trail_.size(); ++i)
   {
     int waypoint_i = std::min(i * stepsize, t->getWayPointCount() - 1);  // limit to last trajectory point
-    auto r = std::make_unique<RobotStateVisualization>(scene_node_, context_,
-                                                       "Trail Robot " + boost::lexical_cast<std::string>(i), nullptr);
+    auto r =
+        std::make_unique<RobotStateVisualization>(scene_node_, context_, "Trail Robot " + std::to_string(i), nullptr);
     r->load(*robot_model_->getURDF());
     r->setVisualVisible(display_path_visual_enabled_property_->getBool());
     r->setCollisionVisible(display_path_collision_enabled_property_->getBool());
@@ -288,8 +294,10 @@ void TrajectoryVisualization::changedTrajectoryTopic()
   if (!trajectory_topic_property_->getStdString().empty() && robot_state_)
   {
     trajectory_topic_sub_ = node_->create_subscription<moveit_msgs::msg::DisplayTrajectory>(
-        trajectory_topic_property_->getStdString(), 2,
-        std::bind(&TrajectoryVisualization::incomingDisplayTrajectory, this, _1));
+        trajectory_topic_property_->getStdString(), rclcpp::SystemDefaultsQoS(),
+        [this](const moveit_msgs::msg::DisplayTrajectory::ConstSharedPtr& msg) {
+          return incomingDisplayTrajectory(msg);
+        });
   }
 }
 
@@ -386,9 +394,9 @@ float TrajectoryVisualization::getStateDisplayTime()
   float value;
   try
   {
-    value = boost::lexical_cast<float>(tm);
+    value = std::stof(tm);
   }
-  catch (const boost::bad_lexical_cast& ex)
+  catch (const std::invalid_argument& ex)
   {
     state_display_time_property_->setStdString(default_time_string);
     return default_time_value;
@@ -408,7 +416,7 @@ void TrajectoryVisualization::dropTrajectory()
   drop_displaying_trajectory_ = true;
 }
 
-void TrajectoryVisualization::update(float wall_dt, float /*ros_dt*/)
+void TrajectoryVisualization::update(float wall_dt, float sim_dt)
 {
   if (drop_displaying_trajectory_)
   {
@@ -419,7 +427,7 @@ void TrajectoryVisualization::update(float wall_dt, float /*ros_dt*/)
   }
   if (!animating_path_)
   {  // finished last animation?
-    boost::mutex::scoped_lock lock(update_trajectory_message_);
+    std::scoped_lock lock(update_trajectory_message_);
 
     // new trajectory available to display?
     if (trajectory_message_to_display_ && !trajectory_message_to_display_->empty())
@@ -440,9 +448,13 @@ void TrajectoryVisualization::update(float wall_dt, float /*ros_dt*/)
       {
         if (static_cast<unsigned int>(trajectory_slider_panel_->getSliderPosition()) >=
             displaying_trajectory_message_->getWayPointCount() - 1)
+        {
           return;  // nothing more to do
+        }
         else
+        {
           animating_path_ = true;
+        }
       }
     }
     trajectory_message_to_display_.reset();
@@ -458,7 +470,14 @@ void TrajectoryVisualization::update(float wall_dt, float /*ros_dt*/)
   {
     int previous_state = current_state_;
     int waypoint_count = displaying_trajectory_message_->getWayPointCount();
-    current_state_time_ += wall_dt;
+    if (use_sim_time_property_->getBool())
+    {
+      current_state_time_ += sim_dt;
+    }
+    else
+    {
+      current_state_time_ += wall_dt;
+    }
     float tm = getStateDisplayTime();
 
     if (trajectory_slider_panel_ && trajectory_slider_panel_->isVisible() && trajectory_slider_panel_->isPaused())
@@ -501,15 +520,19 @@ void TrajectoryVisualization::update(float wall_dt, float /*ros_dt*/)
         trajectory_slider_panel_->setSliderPosition(current_state_);
       display_path_robot_->update(displaying_trajectory_message_->getWayPointPtr(current_state_));
       for (std::size_t i = 0; i < trajectory_trail_.size(); ++i)
+      {
         trajectory_trail_[i]->setVisible(
             std::min(waypoint_count - 1, static_cast<int>(i) * trail_step_size_property_->getInt()) <= current_state_);
+      }
     }
     else
     {
-      animating_path_ = false;       // animation finished
-      if (trajectory_slider_panel_)  // make sure we move the slider to the end
-                                     // so the user can re-play
+      animating_path_ = false;  // animation finished
+      if (trajectory_slider_panel_)
+      {  // make sure we move the slider to the end
+         // so the user can re-play
         trajectory_slider_panel_->setSliderPosition(waypoint_count);
+      }
       display_path_robot_->update(displaying_trajectory_message_->getWayPointPtr(waypoint_count - 1));
       display_path_robot_->setVisible(loop_display_property_->getBool());
       if (!loop_display_property_->getBool() && trajectory_slider_panel_)
@@ -522,7 +545,7 @@ void TrajectoryVisualization::update(float wall_dt, float /*ros_dt*/)
                                    (trajectory_slider_panel_ && trajectory_slider_panel_->isVisible())));
 }
 
-void TrajectoryVisualization::incomingDisplayTrajectory(const moveit_msgs::msg::DisplayTrajectory::ConstSharedPtr msg)
+void TrajectoryVisualization::incomingDisplayTrajectory(const moveit_msgs::msg::DisplayTrajectory::ConstSharedPtr& msg)
 {
   // Error check
   if (!robot_state_ || !robot_model_)
@@ -532,12 +555,14 @@ void TrajectoryVisualization::incomingDisplayTrajectory(const moveit_msgs::msg::
   }
 
   if (!msg->model_id.empty() && msg->model_id != robot_model_->getName())
+  {
     RCLCPP_WARN(LOGGER, "Received a trajectory to display for model '%s' but model '%s' was expected",
                 msg->model_id.c_str(), robot_model_->getName().c_str());
+  }
 
   trajectory_message_to_display_.reset();
 
-  robot_trajectory::RobotTrajectoryPtr t(new robot_trajectory::RobotTrajectory(robot_model_, ""));
+  auto t = std::make_shared<robot_trajectory::RobotTrajectory>(robot_model_, "");
   try
   {
     for (std::size_t i = 0; i < msg->trajectory.size(); ++i)
@@ -563,7 +588,7 @@ void TrajectoryVisualization::incomingDisplayTrajectory(const moveit_msgs::msg::
 
   if (!t->empty())
   {
-    boost::mutex::scoped_lock lock(update_trajectory_message_);
+    std::scoped_lock lock(update_trajectory_message_);
     trajectory_message_to_display_.swap(t);
     if (interrupt_display_property_->getBool())
       interruptCurrentDisplay();
@@ -579,9 +604,13 @@ void TrajectoryVisualization::changedRobotColor()
 void TrajectoryVisualization::enabledRobotColor()
 {
   if (enable_robot_color_property_->getBool())
+  {
     setRobotColor(&(display_path_robot_->getRobot()), robot_color_property_->getColor());
+  }
   else
+  {
     unsetRobotColor(&(display_path_robot_->getRobot()));
+  }
 }
 
 void TrajectoryVisualization::unsetRobotColor(rviz_default_plugins::robot::Robot* robot)
@@ -618,9 +647,19 @@ void TrajectoryVisualization::trajectorySliderPanelVisibilityChange(bool enable)
     return;
 
   if (enable)
+  {
     trajectory_slider_panel_->onEnable();
+  }
   else
+  {
     trajectory_slider_panel_->onDisable();
+  }
+}
+
+void TrajectoryVisualization::clearRobotModel()
+{
+  robot_model_.reset();
+  robot_state_.reset();
 }
 
 }  // namespace moveit_rviz_plugin
